from monai.data import CacheDataset
from monai.data import Dataset
import copy
import glob
import os
from monai.transforms import (
        AddChanneld,
        Resized,
        EnsureChannelFirstd,
        Compose,
        LoadImaged,
        Orientationd,
        ScaleIntensityRanged,
        Spacingd,
        EnsureTyped,
        RandSpatialCropd,
        RandScaleIntensityd,
        RandShiftIntensityd,
        ResizeWithPadOrCropd
)


class WrapDataset(CacheDataset):
    def __init__(self, data=None, transform=None, cache_rate=1.0, num_workers=4):
        super(WrapDataset, self).__init__(data=data, transform=transform, cache_rate=cache_rate, num_workers=num_workers)

    def __getitem__(self, idx):
        data = super().__getitem__(idx)
        return data['image']


class TwoCropWrapDataset(Dataset):
    def __init__(self, data=None, transform_common=None, transform_enhance=1.0, num_workers=4):
        super(TwoCropWrapDataset, self).__init__(data)

        self.data = data
        self.transform_common = transform_common
        self.transform_enhance = transform_enhance


    def __getitem__(self, idx):
        data_i = self.data[idx]
        data_i = self.transform_common(data_i)

        data_ip = copy.deepcopy(data_i)
        data_iq = copy.deepcopy(data_i)

        data_p = self.transform_enhance(data_ip)
        data_q = self.transform_enhance(data_iq)

        return data_p['image'], data_q['image']
        

if __name__ == '__main__':
    data_path = '/dev/shm/aaai2022_pretrained/images'
    train_images = sorted(
        glob.glob(os.path.join(data_path,  "*.nii.gz"))
        )[:2]
    data_dicts = [
        {"image": image_name}
        for image_name in train_images
    ]
    train_files = data_dicts

    xforms_common = [
        LoadImaged(keys=["image"]),
        EnsureChannelFirstd(keys=["image"]),
        Spacingd(keys=["image"], pixdim=(1, 1, 1), mode=("bilinear")),
        Orientationd(keys=["image"], axcodes="RAS"),
        ScaleIntensityRanged(
            keys=["image"], a_min=0, a_max=1700,
            b_min=0.0, b_max=1.0, clip=True,
        ),
        RandSpatialCropd(keys=["image"], roi_size=(224, 224, 224),random_size=False),
        ResizeWithPadOrCropd(keys=["image"],spatial_size=(224, 224, 224)),
    ]

    xforms_enhance = [
        RandScaleIntensityd(keys="image", factors=0.1, prob=0.5),
        RandShiftIntensityd(keys="image", offsets=0.1, prob=0.5),
        EnsureTyped(keys=["image"]),
    ]

    dataset_train = TwoCropWrapDataset(
             data=train_files, 
             transform_common=Compose(xforms_common),
             transform_enhance=Compose(xforms_enhance),
             num_workers=4
    )
    image_p, image_q = dataset_train[0]
    print(image_p.shape)
    print(image_q.shape)